@@ -115,7 +115,7 @@ def encr_data(data, pubkey, bytes=32, output="bytes"):
     key = get_sym_keys(bytes)
     d = encrypt_sym(data, key)
     k = encrypt_asym(key, pubkey)
-    space = b'$$$$'
+    space = b'$$$%%$'
     all = k + space + d
     if output == "hex":
         all = all.hex()
@@ -141,7 +141,7 @@ def decr_data(data, prkey, output="str"):
         pass
     else:
         raise TypeError("data should be str, hex or bytes.")
-    key, da = data.split(b'$$$$')
+    key, da = data.split(b'$$%%$$')
     key = decrypt_asym(key, prkey)
     msg = decrypt_sym(da, key)
     if output == "bytes":
